#ifndef MaterialsH //To avoid linker errors, define header's unique "name" so...
#define MaterialsH //...this header will not be compilled more than one time
//==============================================================================
#include <gl\gl.h>      // Header file for the main OpenGL library
#include <gl\glu.h>     // Header file for the GLu library
//------------------------------------------------------------------------------

//Class TMaterial is a container for data representing material parameters;
//functions for setting them and passing all parameters to OpenGL are also here.

//Remember! All material parameters are passing to OpenGL using glMaterial
//function, so make sure: "glDisable(GL_COLOR_MATERIAL);" was called before

//Declaration of class TMaterial
class TMaterial{
public:
//Class' data (total size = 68 bytes)
float Ambient[4];
float Diffuse[4];
float Specular[4];
float Emission[4];
long int Shininess;
//Default constructor (assigning default values at initialization)
inline TMaterial();
//Functions for setup ambient color
inline void SetAmbient3ub(unsigned char R,unsigned char G,unsigned char B);
inline void SetAmbient4ub(unsigned char R,unsigned char G,unsigned char B,unsigned char A);
inline void SetAmbienti  (int Color);
inline void SetAmbient3f (float R,float G,float B);
inline void SetAmbient3fv(const float *V);
inline void SetAmbient4f (float R,float G,float B,float A);
inline void SetAmbient4fv(const float *V);
//Functions for setup diffuse color
inline void SetDiffuse3ub(unsigned char R,unsigned char G,unsigned char B);
inline void SetDiffuse4ub(unsigned char R,unsigned char G,unsigned char B,unsigned char A);
inline void SetDiffusei  (int Color);
inline void SetDiffuse3f (float R,float G,float B);
inline void SetDiffuse3fv(const float *V);
inline void SetDiffuse4f (float R,float G,float B,float A);
inline void SetDiffuse4fv(const float *V);
//Functions for setup specular color
inline void SetSpecular3ub(unsigned char R,unsigned char G,unsigned char B);
inline void SetSpecular4ub(unsigned char R,unsigned char G,unsigned char B,unsigned char A);
inline void SetSpeculari  (int Color);
inline void SetSpecular3f (float R,float G,float B);
inline void SetSpecular3fv(const float *V);
inline void SetSpecular4f (float R,float G,float B,float A);
inline void SetSpecular4fv(const float *V);
//Functions for setup emission color
inline void SetEmission3ub(unsigned char R,unsigned char G,unsigned char B);
inline void SetEmission4ub(unsigned char R,unsigned char G,unsigned char B,unsigned char A);
inline void SetEmissioni  (int Color);
inline void SetEmission3f (float R,float G,float B);
inline void SetEmission3fv(const float *V);
inline void SetEmission4f (float R,float G,float B,float A);
inline void SetEmission4fv(const float *V);
//Function for setup material's shiness parameter
inline void SetShininessi(long int Value);
//Functions for passing material parameters to OpenGL; suffix "Fast" means that
//ambient value will be used to define both ambient and diffuse parameters
inline void PassAll();
inline void PassAllFast();
inline void PassWithoutEmission();
inline void PassWithoutEmissionFast();
//Functions for loading / writing material data from / to file
inline bool LoadFromMAT(const char* FilePath);
inline bool SaveToMAT(const char* FilePath);
inline void LoadFromChunk(FILE* CurrentlyOpenedFile);
inline void SaveToChunk(FILE* CurrentlyOpenedFile);
};

//============================================================================\\

//Definition of class TMaterial goes here

//Set default OpenGL values for all material parameters at initialization
inline TMaterial::TMaterial(){ //Default constructor
 //Set default values for ambient color
 Ambient[0]=0.2f;
 Ambient[1]=0.2f;
 Ambient[2]=0.2f;
 Ambient[3]=1.0f;
 //Set default values for diffuse color
 Diffuse[0]=0.8f;
 Diffuse[1]=0.8f;
 Diffuse[2]=0.8f;
 Diffuse[3]=1.0f;
 //Set default values for specular color
 Specular[0]=0.0f;
 Specular[1]=0.0f;
 Specular[2]=0.0f;
 Specular[3]=1.0f;
 //Set default shiness parameter
 Shininess=0;
 //Set default values for emission color
 Emission[0]=0.0f;
 Emission[1]=0.0f;
 Emission[2]=0.0f;
 Emission[3]=1.0f;
}

//Ambient color setup functions
inline void TMaterial::SetAmbient3ub(unsigned char R,unsigned char G,unsigned char B){
 Ambient[0] = float(R) / 255.f;
 Ambient[1] = float(G) / 255.f;
 Ambient[2] = float(B) / 255.f;
 Ambient[3] = 1.f;
}
inline void TMaterial::SetAmbient4ub(unsigned char R,unsigned char G,unsigned char B,unsigned char A){
 Ambient[0] = float(R) / 255.f;
 Ambient[1] = float(G) / 255.f;
 Ambient[2] = float(B) / 255.f;
 Ambient[3] = float(A) / 255.f;
}
inline void TMaterial::SetAmbienti(int Color){
 Ambient[0] = float( Color%256 ) / 255.f;
 Ambient[1] = float( (Color/256)%256 ) / 255.f;
 Ambient[2] = float( (Color/65536)%256 ) / 255.f;
 Ambient[3] = 1.f;
}
inline void TMaterial::SetAmbient3f(float R,float G,float B){
 Ambient[0] = R;
 Ambient[1] = G;
 Ambient[2] = B;
 Ambient[3] = 1.f;
}
inline void TMaterial::SetAmbient3fv(const float *V){
 Ambient[0] = V[0];
 Ambient[1] = V[1];
 Ambient[2] = V[2];
 Ambient[3] = 1.f;
}
inline void TMaterial::SetAmbient4f(float R,float G,float B,float A){
 Ambient[0] = R;
 Ambient[1] = G;
 Ambient[2] = B;
 Ambient[3] = A;
}
inline void TMaterial::SetAmbient4fv(const float *V){
 Ambient[0] = V[0];
 Ambient[1] = V[1];
 Ambient[2] = V[2];
 Ambient[3] = V[3];
}

//Diffuse color setup functions
inline void TMaterial::SetDiffuse3ub(unsigned char R,unsigned char G,unsigned char B){
 Diffuse[0] = float(R) / 255.f;
 Diffuse[1] = float(G) / 255.f;
 Diffuse[2] = float(B) / 255.f;
 Diffuse[3] = 1.f;
}
inline void TMaterial::SetDiffuse4ub(unsigned char R,unsigned char G,unsigned char B,unsigned char A){
 Diffuse[0] = float(R) / 255.f;
 Diffuse[1] = float(G) / 255.f;
 Diffuse[2] = float(B) / 255.f;
 Diffuse[3] = float(A) / 255.f;
}
inline void TMaterial::SetDiffusei(int Color){
 Diffuse[0] = float( Color%256 ) / 255.f;
 Diffuse[1] = float( (Color/256)%256 ) / 255.f;
 Diffuse[2] = float( (Color/65536)%256 ) / 255.f;
 Diffuse[3] = 1.f;
}
inline void TMaterial::SetDiffuse3f(float R,float G,float B){
 Diffuse[0] = R;
 Diffuse[1] = G;
 Diffuse[2] = B;
 Diffuse[3] = 1.f;
}
inline void TMaterial::SetDiffuse3fv(const float *V){
 Diffuse[0] = V[0];
 Diffuse[1] = V[1];
 Diffuse[2] = V[2];
 Diffuse[3] = 1.f;
}
inline void TMaterial::SetDiffuse4f(float R,float G,float B,float A){
 Diffuse[0] = R;
 Diffuse[1] = G;
 Diffuse[2] = B;
 Diffuse[3] = A;
}
inline void TMaterial::SetDiffuse4fv(const float *V){
 Diffuse[0] = V[0];
 Diffuse[1] = V[1];
 Diffuse[2] = V[2];
 Diffuse[3] = V[3];
}

//Specular color setup functions
inline void TMaterial::SetSpecular3ub(unsigned char R,unsigned char G,unsigned char B){
 Specular[0] = float(R) / 255.f;
 Specular[1] = float(G) / 255.f;
 Specular[2] = float(B) / 255.f;
 Specular[3] = 1.f;
}
inline void TMaterial::SetSpecular4ub(unsigned char R,unsigned char G,unsigned char B,unsigned char A){
 Specular[0] = float(R) / 255.f;
 Specular[1] = float(G) / 255.f;
 Specular[2] = float(B) / 255.f;
 Specular[3] = float(A) / 255.f;
}
inline void TMaterial::SetSpeculari(int Color){
 Specular[0] = float( Color%256 ) / 255.f;
 Specular[1] = float( (Color/256)%256 ) / 255.f;
 Specular[2] = float( (Color/65536)%256 ) / 255.f;
 Specular[3] = 1.f;
}
inline void TMaterial::SetSpecular3f(float R,float G,float B){
 Specular[0] = R;
 Specular[1] = G;
 Specular[2] = B;
 Specular[3] = 1.f;
}
inline void TMaterial::SetSpecular3fv(const float *V){
 Specular[0] = V[0];
 Specular[1] = V[1];
 Specular[2] = V[2];
 Specular[3] = 1.f;
}
inline void TMaterial::SetSpecular4f(float R,float G,float B,float A){
 Specular[0] = R;
 Specular[1] = G;
 Specular[2] = B;
 Specular[3] = A;
}
inline void TMaterial::SetSpecular4fv(const float *V){
 Specular[0] = V[0];
 Specular[1] = V[1];
 Specular[2] = V[2];
 Specular[3] = V[3];
}

//Emission color setup functions
inline void TMaterial::SetEmission3ub(unsigned char R,unsigned char G,unsigned char B){
 Emission[0] = float(R) / 255.f;
 Emission[1] = float(G) / 255.f;
 Emission[2] = float(B) / 255.f;
 Emission[3] = 1.f;
}
inline void TMaterial::SetEmission4ub(unsigned char R,unsigned char G,unsigned char B,unsigned char A){
 Emission[0] = float(R) / 255.f;
 Emission[1] = float(G) / 255.f;
 Emission[2] = float(B) / 255.f;
 Emission[3] = float(A) / 255.f;
}
inline void TMaterial::SetEmissioni(int Color){
 Emission[0] = float( Color%256 ) / 255.f;
 Emission[1] = float( (Color/256)%256 ) / 255.f;
 Emission[2] = float( (Color/65536)%256 ) / 255.f;
 Emission[3] = 1.f;
}
inline void TMaterial::SetEmission3f(float R,float G,float B){
 Emission[0] = R;
 Emission[1] = G;
 Emission[2] = B;
 Emission[3] = 1.f;
}
inline void TMaterial::SetEmission3fv(const float *V){
 Emission[0] = V[0];
 Emission[1] = V[1];
 Emission[2] = V[2];
 Emission[3] = 1.f;
}
inline void TMaterial::SetEmission4f(float R,float G,float B,float A){
 Emission[0] = R;
 Emission[1] = G;
 Emission[2] = B;
 Emission[3] = A;
}
inline void TMaterial::SetEmission4fv(const float *V){
 Emission[0] = V[0];
 Emission[1] = V[1];
 Emission[2] = V[2];
 Emission[3] = V[3];
}

//Material's shininess setup function
inline void TMaterial::SetShininessi(long int Value){
 Shininess = Value;
}

//Pass all material parameters individually
inline void TMaterial::PassAll(){
 glMaterialfv(GL_FRONT, GL_AMBIENT, Ambient);
 glMaterialfv(GL_FRONT, GL_DIFFUSE, Diffuse);
 glMaterialfv(GL_FRONT, GL_SPECULAR, Specular);
 glMateriali(GL_FRONT, GL_SHININESS, Shininess);
 glMaterialfv(GL_FRONT, GL_EMISSION, Emission);
}

//Pass all material parameters using ambient to define both ambient and diffuse
inline void TMaterial::PassAllFast(){
 glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, Ambient);
 glMaterialfv(GL_FRONT, GL_SPECULAR, Specular);
 glMaterialf(GL_FRONT, GL_SHININESS, Shininess);
 glMaterialfv(GL_FRONT, GL_EMISSION, Emission);
}

//Pass all material parameters (except emission value) individually
inline void TMaterial::PassWithoutEmission(){
 glMaterialfv(GL_FRONT, GL_AMBIENT, Ambient);
 glMaterialfv(GL_FRONT, GL_DIFFUSE, Diffuse);
 glMaterialfv(GL_FRONT, GL_SPECULAR, Specular);
 glMateriali(GL_FRONT, GL_SHININESS, Shininess);
}

//Pass all material parameters (except emission value) using ambient to define
//both ambient and diffuse colors
inline void TMaterial::PassWithoutEmissionFast(){
 glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, Ambient);
 glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, Specular);
 glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, Shininess);
}

//Function loads material definition from file which path given
inline bool TMaterial::LoadFromMAT(const char* FilePath){
 //Try to open specyfied file; if unsuccesfull, return negative result
 FILE* FileHandler=fopen(FilePath,"rb"); if(!FileHandler)return false;
 //Load material definition from file
 fread(this,sizeof(TMaterial),1,FileHandler);
 //Close file
 fclose(FileHandler);
 //As no problems occured, return positive result
 return true;
}

//Function saves material definition to file which path given
inline bool TMaterial::SaveToMAT(const char* FilePath){
 //Create new file or overwrite existing one with the same name
 FILE* FileHandler=fopen(FilePath,"wb"); if(!FileHandler)return false;
 //Write material definition to file
 fwrite(this,sizeof(TMaterial),1,FileHandler);
 //Close file
 fclose(FileHandler);
 return true;
}

//Function loads material definition from currently opened file
inline void TMaterial::LoadFromChunk(FILE* CurrentlyOpenedFile){
 //Load material definition from file
 fread(this,sizeof(TMaterial),1,CurrentlyOpenedFile);
}

//Function saves material definition to currently opened file
inline void TMaterial::SaveToChunk(FILE* CurrentlyOpenedFile){
 //Write material definition to file
 fwrite(this,sizeof(TMaterial),1,CurrentlyOpenedFile);
}

//==============================================================================
#endif //End of header
